@@ -35,6 +35,7 @@
 enum call_type
 {
    CALL_DRAW_VBO,
+   CALL_LAUNCH_GRID,
    CALL_RESOURCE_COPY_REGION,
    CALL_BLIT,
    CALL_FLUSH_RESOURCE,
@@ -77,6 +78,7 @@ struct dd_call
 
    union {
       struct pipe_draw_info draw_vbo;
+      struct pipe_grid_info launch_grid;
       struct call_resource_copy_region resource_copy_region;
       struct pipe_blit_info blit;
       struct pipe_resource *flush_resource;
@@ -88,8 +90,9 @@ struct dd_call
 static FILE *
 dd_get_file_stream(struct dd_context *dctx)
 {
+   struct dd_screen *dscreen = dd_screen(dctx->base.screen);
    struct pipe_screen *screen = dctx->pipe->screen;
-   FILE *f = dd_get_debug_file();
+   FILE *f = dd_get_debug_file(dscreen->verbose);
    if (!f)
       return NULL;
 
@@ -370,6 +373,13 @@ dd_dump_draw_vbo(struct dd_context *dctx, struct pipe_draw_info *info, FILE *f)
    fprintf(f, "\n");
 }
 
+static void
+dd_dump_launch_grid(struct dd_context *dctx, struct pipe_grid_info *info, FILE *f)
+{
+   fprintf(f, "%s:\n", __func__+8);
+   /* TODO */
+}
+
 static void
 dd_dump_resource_copy_region(struct dd_context *dctx,
                              struct call_resource_copy_region *info,
@@ -484,6 +494,9 @@ dd_dump_call(struct dd_context *dctx, struct dd_call *call, unsigned flags)
    case CALL_DRAW_VBO:
       dd_dump_draw_vbo(dctx, &call->info.draw_vbo, f);
       break;
+   case CALL_LAUNCH_GRID:
+      dd_dump_launch_grid(dctx, &call->info.launch_grid, f);
+      break;
    case CALL_RESOURCE_COPY_REGION:
       dd_dump_resource_copy_region(dctx, &call->info.resource_copy_region, f);
       break;
@@ -588,8 +601,11 @@ dd_context_flush(struct pipe_context *_pipe,
 static void
 dd_before_draw(struct dd_context *dctx)
 {
-   if (dd_screen(dctx->base.screen)->mode == DD_DETECT_HANGS &&
-       !dd_screen(dctx->base.screen)->no_flush)
+   struct dd_screen *dscreen = dd_screen(dctx->base.screen);
+
+   if (dscreen->mode == DD_DETECT_HANGS &&
+       !dscreen->no_flush &&
+       dctx->num_draw_calls >= dscreen->skip_count)
       dd_flush_and_handle_hang(dctx, NULL, 0,
                                "GPU hang most likely caused by internal "
                                "driver commands");
@@ -598,22 +614,34 @@ dd_before_draw(struct dd_context *dctx)
 static void
 dd_after_draw(struct dd_context *dctx, struct dd_call *call)
 {
-   switch (dd_screen(dctx->base.screen)->mode) {
-   case DD_DETECT_HANGS:
-      if (!dd_screen(dctx->base.screen)->no_flush &&
-          dd_flush_and_check_hang(dctx, NULL, 0)) {
-         dd_dump_call(dctx, call, PIPE_DEBUG_DEVICE_IS_HUNG);
+   struct dd_screen *dscreen = dd_screen(dctx->base.screen);
+   struct pipe_context *pipe = dctx->pipe;
 
-         /* Terminate the process to prevent future hangs. */
-         dd_kill_process();
+   if (dctx->num_draw_calls >= dscreen->skip_count) {
+      switch (dscreen->mode) {
+      case DD_DETECT_HANGS:
+         if (!dscreen->no_flush &&
+            dd_flush_and_check_hang(dctx, NULL, 0)) {
+            dd_dump_call(dctx, call, PIPE_DEBUG_DEVICE_IS_HUNG);
+
+            /* Terminate the process to prevent future hangs. */
+            dd_kill_process();
+         }
+         break;
+      case DD_DUMP_ALL_CALLS:
+         if (!dscreen->no_flush)
+            pipe->flush(pipe, NULL, 0);
+         dd_dump_call(dctx, call, 0);
+         break;
+      default:
+         assert(0);
       }
-      break;
-   case DD_DUMP_ALL_CALLS:
-      dd_dump_call(dctx, call, 0);
-      break;
-   default:
-      assert(0);
    }
+
+   ++dctx->num_draw_calls;
+   if (dscreen->skip_count && dctx->num_draw_calls % 10000 == 0)
+      fprintf(stderr, "Gallium debugger reached %u draw calls.\n",
+              dctx->num_draw_calls);
 }
 
 static void
@@ -632,6 +660,22 @@ dd_context_draw_vbo(struct pipe_context *_pipe,
    dd_after_draw(dctx, &call);
 }
 
+static void
+dd_context_launch_grid(struct pipe_context *_pipe,
+                       const struct pipe_grid_info *info)
+{
+   struct dd_context *dctx = dd_context(_pipe);
+   struct pipe_context *pipe = dctx->pipe;
+   struct dd_call call;
+
+   call.type = CALL_LAUNCH_GRID;
+   call.info.launch_grid = *info;
+
+   dd_before_draw(dctx);
+   pipe->launch_grid(pipe, info);
+   dd_after_draw(dctx, &call);
+}
+
 static void
 dd_context_resource_copy_region(struct pipe_context *_pipe,
                                 struct pipe_resource *dst, unsigned dst_level,
@@ -773,6 +817,7 @@ dd_init_draw_functions(struct dd_context *dctx)
 {
    CTX_INIT(flush);
    CTX_INIT(draw_vbo);
+   CTX_INIT(launch_grid);
    CTX_INIT(resource_copy_region);
    CTX_INIT(blit);
    CTX_INIT(clear);
